@@ -159,7 +159,7 @@ print.breakpoints <- function(x, format.times = NULL, ...)
             "-segment partition: \n\n", sep = ""))
   cat("Call:\n")
   print(x$call)
-  cat("\nBreakpoints at obervation number:\n")
+  cat("\nBreakpoints at observation number:\n")
   cat(x$breakpoints,"\n")
   cat("\nCorresponding to breakdates:\n")
   cat(breakdates(x, format.times = format.times),"\n")
@@ -185,7 +185,7 @@ breakdates.breakpoints <- function(obj, format.times = FALSE, breaks = NULL, ...
     return(RVAL)
   }
 
-  if(is.na(obj$breakpoints))
+  if(is.na(obj$breakpoints)[1])
     breakdates <- NA
   else {
     breakdates <- (obj$breakpoints - 1)/obj$datatsp[3] + obj$datatsp[1]
@@ -284,7 +284,7 @@ print.summary.breakpointsfull <- function(x, ...)
   cat("\n\t Optimal (m+1)-segment partition: \n\n")
   cat("Call:\n")
   print(x$call)
-  cat("\nBreakpoints at obervation number:\n")
+  cat("\nBreakpoints at observation number:\n")
   print(bp, quote = FALSE)
   cat("\nCorresponding to breakdates:\n")
   print(bd, quote = FALSE)
@@ -369,18 +369,17 @@ pargmaxV <- function(x, xi = 1, phi1 = 1, phi2 = 1)
   ifelse(x < 0, G1(x, xi = xi, phi = phi), G2(x, xi = xi, phi = phi))
 }
 
-confint <- function(object, level = 0.95, ...)
-{
-  UseMethod("confint")
-}
-
-confint.breakpointsfull <- function(object, level = 0.95, breaks = NULL,
+confint.breakpointsfull <- function(object, parm = NULL, level = 0.95, breaks = NULL,
                                     het.reg = TRUE, het.err = TRUE, ...)
 {
   X <- object$X
   y <- object$y
   n <- object$nobs
   a2 <- (1 - level)/2
+  if(!is.null(parm) & !is.null(breaks))
+    warning("`parm' and `breaks' are both specified: `breaks' is used")
+  else
+    if(!is.null(parm)) breaks <- parm
 
   myfun <- function(x, level = 0.975, xi = 1, phi1 = 1, phi2 = 1)
     (pargmaxV(x, xi = xi, phi1 = phi1, phi2 = phi2) - level)^2
@@ -470,7 +469,7 @@ print.confint.breakpoints <- function(x, format.times = NULL, ...)
   cat(paste("\n\t of optimal ", nbp + 1, "-segment partition: \n\n", sep = ""))
   cat("Call:\n")
   print(x$call)
-  cat("\nBreakpoints at obervation number:\n")
+  cat("\nBreakpoints at observation number:\n")
   print(x$confint, quote = FALSE)
   cat("\nCorresponding to breakdates:\n")
   print(breakdates(x, format.times = format.times, ...), quote = FALSE)
